'use strict';

module.exports = {
  label: 'Tasks',
  nodes: [
    {
      label: 'noop',
      type: 'task',
      nodes: [],
    },
    {
      label: 'fn1',
      type: 'task',
      nodes: [],
    },
    {
      label: 'fn2',
      type: 'task',
      nodes: [],
    },
    {
      label: 'fn3',
      type: 'task',
      nodes: [],
    },
    {
      label: 'fn4',
      type: 'task',
      nodes: [],
    },
  ],
};
